set term ^;

create or alter procedure TMP_REMAKE_TAXATION
AS
declare variable id_sale_taxation integer;
declare variable prev_id_sale_taxation integer;
declare variable old_id_sale_tax integer;
declare variable rev_tax integer;
declare variable new_id_sale_taxation integer;
declare variable tax_name varchar(50);
declare variable dl_dbkey char(8);
declare variable ap_dbkey char(8);
declare variable nat_round_to numeric(15,4);
declare variable line_percent numeric(15,4);
declare variable tax_list varchar(250);
declare variable id_base_doc integer;
declare variable doc_selected integer;
declare variable id_doc_line integer;
declare variable id_account_post integer;
declare variable id_hotel integer;
declare variable ac varchar(2);
declare variable rev_tax_name varchar(50);
begin
  doc_selected = -gen_id(DOC_SELECTED_GEN, 1);

  new_id_sale_taxation = null;
  prev_id_sale_taxation = -1;
  for
    select dl.RDB$DB_KEY, dl.ID_SALE_TAXATION, dl.VAT_PERCENT, d.ID_BASE_DOC, dl.ID_DOC_LINE,
           a.ID_HOTEL, c.ROUND_TO
    from   DOC_HEAD d
           join DOC_LINE dl on dl.ID_DOC_HEAD = d.ID_DOC
           join ACCOUNT a on a.ID_ACCOUNT = d.ID_ACCOUNT
           join HOTEL h on h.ID_HOTEL = a.ID_HOTEL
           join CURRS c on c.CURR = h.NATION_CURR
    where  exists
           (select stt.ID_SALE_TAX
            from   SALE_TAXATION_TAX stt
                   join SALE_TAXES st on st.ID_SALE_TAX = stt.ID_SALE_TAX
            where  stt.ID_SALE_TAXATION = dl.ID_SALE_TAXATION
                   and st.REVENUE_DEDUCTION_TAX is distinct from 1
                   and not exists (select dt.TAX_PERCENT
                                   from   DOC_TAXES dt
                                   where  dt.ID_DOC_HEAD = d.ID_DOC
                                          and dt.ID_SALE_TAX = stt.ID_SALE_TAX))
    order by 2 asc
    into   :dl_dbkey, :id_sale_taxation, :line_percent, :id_base_doc, :id_doc_line,
           :id_hotel, :nat_round_to
  do begin
    if (:prev_id_sale_taxation <> :id_sale_taxation) then begin
      tax_list = '';
      select first 1 st.ID_SALE_TAX, st.TAX_NAME
      from   SALE_TAXES st
      where  st.TAX_PERCENT = :line_percent
             and st.ID_HOTEL = :id_hotel
      order by st.ID_SALE_TAX desc
      into   :old_id_sale_tax, :tax_name;

      tax_list = :old_id_sale_tax;

      for
        select st.ID_SALE_TAX, st.TAX_NAME
        from   SALE_TAXATION_TAX t
               join SALE_TAXES st on st.ID_SALE_TAX = t.ID_SALE_TAX
        where  t.ID_SALE_TAXATION = :id_sale_taxation
               and st.REVENUE_DEDUCTION_TAX = 1
        into   :rev_tax, :rev_tax_name
      do begin
        tax_list = :tax_list || ',' || :rev_tax;
        tax_name = :tax_name || ',' || substring(:rev_tax_name from 1 for 3);
      end
      tax_name = substring(:tax_name from 1 for 40) || ' - archive';
      if (exists (select stt.ID_SALE_TAXATION
                  from   SALE_TAXATION stt
                  where  stt.NAME = :tax_name and stt.ID_HOTEL = :id_hotel)) then
        tax_name = substring(:tax_name from 1 for 45) || :id_sale_taxation;

      execute procedure SALE_TAXATION_MODIFY$CFG (null, :tax_name , :tax_list, '', 'add', :id_hotel)
      returning_values (:new_id_sale_taxation);
    
    end

    update DOC_LINE dl
    set    dl.ID_SALE_TAXATION = :new_id_sale_taxation
    where  dl.RDB$DB_KEY = :dl_dbkey;

    select ap.RDB$DB_KEY, ap.AC, ap.ID_ACCOUNT_POST
    from   DOC_LINE_POST_VALUES dlpv
           join ACCOUNT_POST ap on ap.ID_ACCOUNT_POST = dlpv.ID_ACCOUNT_POST
    where  dlpv.ID_DOC_LINE = :id_doc_line
           and ap.ID_BASE_DOC = :id_base_doc
           and ap.ID_SALE_TAXATION = :id_sale_taxation
    into   :ap_dbkey, :ac, :id_account_post;

    if (:ap_dbkey is not null) then begin
      if (:ac = '00') then
        execute procedure ACCOUNT_POSTING_UPDATE_TAX$INT (:ap_dbkey, :new_id_sale_taxation, :nat_round_to, 'SERVICE VAT CHANGED');
      else
        update ACCOUNT_POST AP
        set    ap.DOC_SELECTED = :doc_selected,
               ap.ID_SALE_TAXATION = :new_id_sale_taxation
        where  ap.RDB$DB_KEY = :ap_dbkey;

      update ACCOUNT_DEBT ad
      set    ad.ID_SALE_TAXATION = :new_id_sale_taxation
      where  ad.ID_ACCOUNT_POST = :id_account_post
             and ad.ID_SALE_TAXATION = :id_sale_taxation;
    end
    ap_dbkey = null;
    prev_id_sale_taxation = :id_sale_taxation;
  end

end
^


set term ;^


execute procedure TMP_REMAKE_TAXATION;

commit work;

drop procedure TMP_REMAKE_TAXATION;

commit work;



