set term ^;


CREATE OR ALTER PROCEDURE DOCUMENT_ANUL$DOC (
    ID_DOC_HEAD INTEGER,
    ANUL_REASON VARCHAR(100),
    MAKE_ANNUL_REPORT SMALLINT)
AS
/**/
  declare variable id_account  integer;
  declare variable id_payment_head integer;
  declare variable for_deposit integer;
  declare variable id_base_doc integer;
  declare variable is_anul integer;
  declare variable dh_dbkey char(8);
  declare variable id_annul_protocol_num integer;
  declare variable num integer;
  declare variable doc_num varchar(20);
  declare variable system_date date;
  declare variable id_reserv_detail integer;
  declare variable check_status integer;
  declare variable doc_date date;
  declare variable id_system_log integer;
  declare variable id_hotel integer;
  declare variable is_storno_doc integer;
  declare variable no_use integer;
begin

  select H.RDB$DB_KEY, H.ID_ACCOUNT, H.FOR_DEPOSIT, H.ID_BASE_DOC, H.IS_ANUL, A.ID_HOTEL,
         A.ID_RESERV_DETAIL, T.ID_ANNUL_DOC_NUM, h.DOC_DATE, H.IS_STORNO
  from   DOC_HEAD$INT H
         join DOC_COMPLETION dc on dc.ID_DOC_HEAD = h.ID_DOC_HEAD
         join DOC_TYPE T on H.ID_DOC_TYPE = T.ID_DOC_TYPE
         join DOC_TYPE_INV DTI on DTI.ID_DOC_TYPE = T.ID_DOC_TYPE
         join ACCOUNT A on A.ID_ACCOUNT = H.ID_ACCOUNT
  where  H.ID_DOC_HEAD = :ID_DOC_HEAD
  into   :dh_dbkey, :id_account, :for_deposit, :id_base_doc, :is_anul, :id_hotel,
         :id_reserv_detail, :id_annul_protocol_num, :doc_date, :is_storno_doc;

  if (:is_anul = 1) then
    exception DOCUMENT_ANNUL_EXCEPTION '-352'; --'Document is already annul';

  if (exists (select D.ID_DOC
              from   DOC_HEAD D
              where  D.ID_BASE_DOC = :ID_BASE_DOC
                     and D.ID_DOC > :ID_DOC_HEAD
                     and D.IS_ANUL = 0))
  then exception DOCUMENT_ANNUL_EXCEPTION '-314';

  execute procedure SYSTEM_DATE_GET(:ID_HOTEL)
  returning_values  :system_date;

  if (:DOC_DATE < :SYSTEM_DATE) then begin
    execute procedure RIGHTS_CHECK_GRANTED$INT ('DOC_ANNUL_PREV_DATE');
  end
  /* unarchive account and set as credit account, for shecked out bookings only */
  execute procedure ACCOUNT_SET_ARCHIVE_STATUS(:ID_ACCOUNT, 0);

  if (:ID_RESERV_DETAIL is not null) then begin
    select RD.CHECK_STATUS
    from   RESERV_DETAIL RD
    where  RD.ID_RESERV_DETAIL = :ID_RESERV_DETAIL
    into   :check_status;

    if (:check_status = 1) then begin
      execute procedure ACCOUNT_SET_CREDIT_STATUS$INT (:id_account, 1, null, 1);
    end
  end
  /* MARK THE DOCUMENT AS ANUL - before posting and payment updates */
  update DOC_HEAD$INT DH
  set    DH.IS_ANUL = 1
  where  DH.RDB$DB_KEY = :dh_dbkey;

  update DOC_COMPLETION Dc
  set    Dc.IS_EFAKTURA_EXPORTED = 0
  where  Dc.ID_DOC_HEAD = :ID_DOC_HEAD;


  if (:is_storno_doc = 1) then begin
    /* annul storno doc */
    update DOC_HEAD$INT BD
    set    BD.IS_STORNED = 0
    where  BD.ID_DOC_HEAD = :id_base_doc
           and BD.IS_STORNED = 1;
  end
  /* RELEASE ACCOUNT_POSTS */
  if (:for_deposit = 0) then
    execute procedure ACCOUNT_POSTING_DOC_UPDATE$INT (:DH_DBKEY)
    returning_values (:no_use);

  /* PAYMENTS REFLECTION */
  for
    select PH.ID_PAYMENT_HEAD
    from   PAYMENT_HEAD PH
           join DOC_TYPE_PAY dt on dt.ID_DOC_TYPE = ph.ID_DOC_TYPE
    where  PH.ID_DOC_HEAD = :ID_DOC_HEAD
           and PH.IS_STORNED = 0
    into   :ID_PAYMENT_HEAD
  do begin

    execute procedure PAYMENT_ANUL$INT (:ID_PAYMENT_HEAD, :ANUL_REASON);
  end

  /* ANNUL_PROTOCOL creating section */
  NUM = null;
  DOC_NUM = null;
  if (:MAKE_ANNUL_REPORT = 1) then begin
    execute procedure RIGHTS_CHECK_GRANTED$INT ('DOC_ANNUL_WITH_PROT');

    if (:ID_ANNUL_PROTOCOL_NUM is null) then exception DOCUMENT_ANNUL_EXCEPTION '-345';

    execute procedure SYSTEM_GEN_NUM ('ANNUL_PROT', :ID_ANNUL_PROTOCOL_NUM)
    returning_values  (:num, :doc_num);
  end
  /* log annul event */
  execute procedure SYSTEM_EVENT_LOG$INT ('DOCUMENT', :ID_DOC_HEAD, 'ANNUL', f_character(29)||:ANUL_REASON)
  returning_values  :id_system_log;

  insert into DOC_ANNULMENT (ID_DOC_HEAD, ANNULMENT_DATE, ID_PROT_DOC_NUM, PROT_NUM, PROT_DOC_NUM, ID_LOG_EVENT)
  values (:ID_DOC_HEAD, :SYSTEM_DATE, :ID_ANNUL_PROTOCOL_NUM, :NUM, :DOC_NUM, :ID_SYSTEM_LOG);
  /* reflect annul date on doc release dates */
  if (:DOC_DATE < :SYSTEM_DATE) then begin
    insert into DOC_RELEASE_DATE (ID_DOC_HEAD, RELEASE_DATE, VALUE_CORR_INDEX)
    values (:ID_DOC_HEAD, :SYSTEM_DATE, -1);
  end else begin
    update DOC_RELEASE_DATE RD
    set    RD.VALUE_CORR_INDEX = 0
    where  RD.ID_DOC_HEAD = :ID_DOC_HEAD
           and RD.RELEASE_DATE = :DOC_DATE;
  end
  /* reflect on AR module */
  execute procedure DOCUMENT_AR_PAYMENTS_APPLY$INT (:DH_DBKEY, null, 1, :ANUL_REASON);
  /* reflect on client loyalty module */
  execute procedure DOCUMENT_BONUSPOINTS_VOID$CL (:ID_DOC_HEAD);

end
^

CREATE OR ALTER PROCEDURE ACCOUNT_POSTING_SELECT$DOC (
    ID_ACCOUNT INTEGER,
    IN_DOC_SELECTED INTEGER,
    TO_DATE DATE = null)
RETURNS (
    OUT_DOC_SELECTED INTEGER)
AS
declare variable id_deposit integer;
begin

  OUT_DOC_SELECTED = coalesce (:IN_DOC_SELECTED, gen_id(DOC_SELECTED_GEN, 1));

  update ACCOUNT_POST AP
  set    AP.DOC_SELECTED = :OUT_DOC_SELECTED
  where  AP.ID_ACCOUNT = :ID_ACCOUNT
         and AP.AC = '00'
         and AP.DATE_POST <= coalesce(:TO_DATE, AP.DATE_POST);
  /*
  for
    select DA.ID_DEPOSIT
    from   DEPOSIT_AVAILABLE DA
    where  DA.ID_ACCOUNT = :ID_ACCOUNT
    into   :id_deposit
  do begin
    update DEPOSIT D
    set    D.DOC_SELECTED = :OUT_DOC_SELECTED
    where  D.ID_DEPOSIT = :id_deposit;
  end
  */
end
^

CREATE OR ALTER PROCEDURE DOCUMENT_ADVANCE_PAYMENT$INT (
    ID_DOC_HEAD INTEGER,
    ID_PAYMENT_HEAD INTEGER,
    ACC_RATE NUMERIC(15,6))
AS
declare variable id_deposit integer;
  declare variable dep_curr varchar(3);
  declare variable pay_rate numeric(15,6);
  declare variable id_doc_line integer;
  declare variable doc_value numeric(15,4);
  declare variable init_pay_value numeric(15,4);
  declare variable deducted_value numeric(15,4);
  declare variable pay_round_to numeric(15,4);
  declare variable id_account_payment integer;
  declare variable initial_deposit_id integer;
  declare variable pay_value numeric(15,4);
  declare variable total_pay_value numeric(15,4);
  declare variable total_doc_value numeric(15,4);
  declare variable id_dep_doc integer;
begin
  total_pay_value = 0;
  total_doc_value = 0;
  for
    select D.ID_DEPOSIT, D.PAY_CURRENCY, d.INIT_PAY_AMOUNT, d.DEDUCTED_AMOUNT, d.ID_DOC_HEAD
    from   DEPOSITS D
    where  d.ID_PAYMENT_HEAD = :id_payment_head
    order by d.ID_DEPOSIT asc
    into   :id_deposit, :dep_curr, :init_pay_value, :deducted_value, :id_dep_doc
  do begin
    initial_deposit_id = coalesce(:initial_deposit_id, :id_deposit);
    if (coalesce(:deducted_value, 0) > 0) then
      exception DOCUMENT_EXCEPTION '-350';

    if (:id_dep_doc is not null) then
      exception DOCUMENT_EXCEPTION '-364';

    if (:pay_rate is null) then
      execute procedure DOCUMENT_CURR_RATE_GET$INT (:ID_DOC_HEAD, :dep_curr)
      returning_values  (:pay_rate, :pay_round_to);
    /*
    if (round_f(:dep_value, :pay_round_to) <= 0) then
      exception DOCUMENT_EXCEPTION 'Unavailable payment amount!';
    */
    update ACCOUNT_PAYMENTS AP
    set    AP.ACC_AMOUNT = AP.PAY_AMOUNT * cast(:pay_rate / :acc_rate as double precision),
           Ap.ACC_RATE  = :acc_rate
    where  AP.ID_DEPOSIT = :id_deposit;

    select min(ap.ID_ACCOUNT_PAYMENT), sum(ap.PAY_AMOUNT)
    from   ACCOUNT_PAYMENTS AP
    where  AP.ID_DEPOSIT = :id_deposit
    into   :id_account_payment, :pay_value;
    total_pay_value = :total_pay_value + :pay_value;

    execute procedure DOCUMENT_ATTACH_PAYMENT$INT (:ID_DOC_HEAD, :id_account_payment, :pay_rate, :pay_value)
    returning_values (:doc_value);
    total_doc_value = :total_doc_value + :doc_value;
  end

  if (:total_pay_value <> 0) then begin
    if (round_f(:total_pay_value - :init_pay_value, :pay_round_to) <> 0) then
      exception DOCUMENT_EXCEPTION '-390' || f_character(31)
                || f_character(29) || formatfloat(:total_pay_value, 'F;0.00')
                || f_character(28) || formatfloat(:init_pay_value, 'F;0.00');
  
    execute procedure DOCUMENT_ADD_PAYMENT_LINE$INT (:initial_deposit_id, :ID_DOC_HEAD, :total_doc_value)
    returning_values  (:id_doc_line);

    update PAYMENT_HEAD P
    set    P.ID_DOC_HEAD = :ID_DOC_HEAD
    where  P.ID_PAYMENT_HEAD = :id_payment_head;
  end
end
^

create or alter procedure EXPORT_HK_PLAN_LINES$EXC (
    ID_PLAN_PERIOD type of ID)
returns (
    USE_PER_ARRIVAL type of TLOGICAL,
    USE_PER_DEPARTURE type of TLOGICAL,
    CYCLE_DURATION type of TRANGE,
    QTY type of TCOUNT,
    MULTIPLY_TYPE type of NAME_20,
    ITEM_NAME type of TOBJECT_NAME,
    TYPE_ROOM_LIST type of TTEXT_EXTENDED)
AS
declare variable ID_PLAN_LINE INTEGER;
declare variable TYPE_ROOM VARCHAR(20);
begin
  for
    select pl.ID_PLAN_LINE, pl.USE_PER_ARRIVAL, pl.USE_PER_DEPARTURE,
           pl.CYCLE_DURATION, pl.QTY, pl.MULTIPLY_TYPE, pi.ITEM_NAME
    from   HK_PLAN_LINE pl
           join HK_PLAN_ITEM pi on pl.ID_PLAN_ITEM = pi.ID_PLAN_ITEM
    where  pl.ID_PLAN_PERIOD = :ID_PLAN_PERIOD
    into   :ID_PLAN_LINE, :USE_PER_ARRIVAL, :USE_PER_DEPARTURE,
           :CYCLE_DURATION, :QTY, :MULTIPLY_TYPE, :ITEM_NAME
  do begin
    TYPE_ROOM_LIST = '';
    for
      select rt.TYPE_ROOM
      from   HK_PLAN_ROOM_TYPE pr
             join ROOM_TYPES rt on pr.ID_ROOM_TYPE = rt.ID_ROOM_TYPE
      where  pr.ID_PLAN_LINE = :ID_PLAN_LINE
      into   :TYPE_ROOM
    do begin
      TYPE_ROOM_LIST = :TYPE_ROOM_LIST || :TYPE_ROOM || ',';
    end
    suspend;
  end
end
^

set term ;^

