SET TERM ^ ;

CREATE OR ALTER procedure ACCOUNT_TAXATION_RECALC$INT (
    ID_ACCOUNT integer,
    NEW_ID_TAX_POLICY integer)
AS
declare variable nat_round_to numeric(15,4);
  declare variable ap_dbkey char(8);
  declare variable id_account_sale_taxation integer;
  declare variable id_sale_taxation integer;
  declare variable nat_curr varchar(3);
  declare variable id_service integer;
  declare variable id_rate integer;
  declare variable passed_nights integer;
begin
  select h.NATION_CURR
  from   ACCOUNT A
         join HOTEL H on h.ID_HOTEL = a.ID_HOTEL
  where  a.ID_ACCOUNT = :ID_ACCOUNT
  into   :nat_curr;

  select C.ROUND_TO
  from   CURRS C
  where  c.CURR = :nat_curr
  into   :nat_round_to;

  for
    select ap.RDB$DB_KEY, ST.ID_SALE_TAXATION, ap.ID_SERVICE, ap.ID_TARIFF, ap.DATE_POST - ap.FROM_DATE
    from   ACCOUNT_POST AP
           join SERVICE_TAXATION ST on ST.ID_SERVICE = ap.ID_SERVICE
    where  AP.ID_ACCOUNT = :ID_ACCOUNT
           and AP.ID_BASE_DOC is null
           and ST.ID_TAX_POLICY = :NEW_ID_TAX_POLICY
           and ap.IS_ANUL = 0
    into   :ap_dbkey, :id_account_sale_taxation, :id_service, :id_rate, :passed_nights
  do begin
    execute procedure ACCOUNT_POSTING_TAXATION_GET (:id_service, :NEW_ID_TAX_POLICY, :id_rate, :passed_nights)
    returning_values (:id_sale_taxation);

    if (:id_account_sale_taxation = :id_sale_taxation) then begin
      -- there wasn't different (explicit) taxation from rating
      execute procedure ACCOUNT_POSTING_UPDATE_TAX$INT (:ap_dbkey, :id_account_sale_taxation,
              :nat_round_to, 'ACCOUNT VAT POLICY CHANGED');
    end
  end
end
^


CREATE OR ALTER procedure SERVICE_TAXATION_MODIFY$CFG (
    ID_TAX_POLICY type of ID,
    ID_SERVICE type of ID,
    ID_SALE_TAXATION type of ID)
AS
declare variable dbkey char(8);
  declare variable nat_round_to numeric(15,4);
  declare variable nat_curr varchar(3);
  declare variable action_text varchar(255);
  declare variable old_sale_taxation_name varchar(50);
  declare variable service_name varchar(30);
  declare variable id_hotel integer;
begin
  if (:ID_SALE_TAXATION is null) then begin
    delete from SERVICE_TAXATION T
    where  t.ID_TAX_POLICY = :ID_TAX_POLICY
           and t.ID_SERVICE = :ID_SERVICE;

    select f_character(29) || se.service_name
    from   SERVICES se
    where  se.id_service = :id_service
    into   :action_text;
    execute procedure SYSTEM_LOG_ADD ('TAX_POLICY', :id_tax_policy, 'TAX_POLICY_SERVICE_DELETED', :action_text);
  end else begin
    select sta.name, se.service_name, se.ID_HOTEL
    from   SERVICE_TAXATION TPT
           join SALE_TAXATION sta on sta.id_sale_taxation = tpt.id_sale_taxation
           join SERVICES se on se.id_service = tpt.id_service
    where  TPT.ID_TAX_POLICY = :ID_TAX_POLICY
           and TPT.ID_SERVICE = :ID_SERVICE
    into   :old_sale_taxation_name, :service_name, :id_hotel;

    update SERVICE_TAXATION T
    set    T.ID_SALE_TAXATION = :ID_SALE_TAXATION
    where  T.ID_TAX_POLICY = :ID_TAX_POLICY
           and T.ID_SERVICE = :ID_SERVICE;

    if (row_count > 0) then begin
      select st.name
      from   SALE_TAXATION st
      where  st.id_sale_taxation = :id_sale_taxation
      into   :action_text;
      action_text = f_character(29) || :service_name || f_character(28) || :old_sale_taxation_name || f_character(28) || :action_text;
      execute procedure SYSTEM_LOG_ADD ('TAX_POLICY', :id_tax_policy, 'TAX_POLICY_SERVICE_UPDATED', :action_text);
    end else begin
      insert into SERVICE_TAXATION (ID_TAX_POLICY, ID_SERVICE, ID_SALE_TAXATION)
      values (:ID_TAX_POLICY, :ID_SERVICE, :ID_SALE_TAXATION);
    end
  
    select h.NATION_CURR
    from   HOTEL H
    where  h.ID_HOTEL = :id_hotel
    into   :nat_curr;

    select C.ROUND_TO
    from   CURRS C
    where  c.CURR = :nat_curr
    into   :nat_round_to;
  
    for
      select AP.RDB$DB_KEY
      from   ACCOUNT A
             join ACCOUNT_POST AP on AP.ID_ACCOUNT = A.ID_ACCOUNT
      where  A.ID_TAX_POLICY = :ID_TAX_POLICY
             and A.IS_ARCHIVE = 0
             and AP.ID_SERVICE = :ID_SERVICE
             and AP.ID_BASE_DOC is null
             and AP.ID_SALE_TAXATION is distinct from :ID_SALE_TAXATION
             and ap.IS_ANUL = 0
      into   :dbkey
    do begin
      execute procedure ACCOUNT_POSTING_UPDATE_TAX$INT (:dbkey, :ID_SALE_TAXATION, :nat_round_to, 'SERVICE VAT CHANGED');
    end
  end
end
^


ALTER trigger ACCOUNT_POST_CORR_LOG
active after update position 2
AS 
  declare variable is_positive integer;
  declare variable is_anul integer;
  declare variable is_open integer;
  declare variable sys_date date;
  declare variable is_unanul integer;
  declare variable is_price_correct integer;
  declare variable qty_diff numeric(15,2);
  declare variable price_diff numeric(15,4);
  declare variable value_diff numeric(15,4);
  declare variable rate_diff numeric(15,6);
  declare variable old_qty_real numeric(15,2);
  declare variable new_qty_real numeric(15,2);
  declare variable reason_of_correct varchar(30);
  declare variable datetime_created timestamp;
  declare variable min_price_diff numeric(15,4);
  declare variable min_rate_diff numeric(15,6);
  declare variable min_qty_diff numeric(15,2);
  declare variable id_hotel integer;
  declare variable is_value_diff smallint;
begin
  if (old.DOC_SELECTED <> new.DOC_SELECTED) then
    /* IF NEW.DOC_SELECTED IS NEGATIVE THEN NEW DOCUMENT IS RELEASING.
    || ELSE ACCOUNT_POST IS MARKING FOR DOCUMENT
    || THIS IS ARTIFICIAL WAY TO SUPPRESS EXECUTION OF MAIN PART THIS TRIGGER
    */
    exit;
  else begin
    if (old.IS_ANUL = 1 and new.IS_ANUL = 1) then exit;  -- doesn't trace annul posting changes
    /* real update  */
    select A.ID_HOTEL
    from   ACCOUNT A
    where  A.ID_ACCOUNT = old.ID_ACCOUNT
    into   :id_hotel;

    execute procedure SYSTEM_DATE_GET(:id_hotel)
    returning_values :sys_date;
    min_qty_diff   = 0.01;     /* may be 1 ?! */
    min_price_diff = 0.0001;   /* because the price is roundless */
    min_rate_diff  = 0.00001;  /* the EUR's rate is with 5 digits after decimal symbol */
    /* calculate qty and value diffs for loging */
    is_anul   = (new.IS_ANUL  -  old.IS_ANUL) * new.IS_ANUL;
    is_unanul = (old.IS_ANUL  -  new.IS_ANUL) * old.IS_ANUL;
    old_qty_real = old.QTY_POST * (1 - :is_unanul);
    new_qty_real = new.QTY_POST * (1 - :is_anul);
    qty_diff     = round_f(:new_qty_real  - :old_qty_real, :min_qty_diff);
    price_diff   = round_f(new.PRICE_POST - old.PRICE_POST, :min_price_diff);
    rate_diff    = round_f(new.RATE_POST  - old.RATE_POST, :min_rate_diff);
    is_value_diff = iif((:price_diff <> 0) or (:rate_diff <> 0), 1, 0);
    value_diff    = round_f(new.PRICE_POST * new.RATE_POST - old.PRICE_POST * old.RATE_POST, :min_price_diff);
    /*   */
    is_open = iif(new.ID_BASE_DOC is not null and new.AC = '00', 1, 0);
    reason_of_correct = iif(:is_anul = 0, new.CHANGED_TEXT, new.ANUL_REASON);
    datetime_created = current_timestamp;
    /* log service or account change  */
    if (   (old.ID_SERVICE is distinct from new.ID_SERVICE)    /*  service change           */
        or (old.ID_ACCOUNT is distinct from new.ID_ACCOUNT)    /*  posting transfer         */
        or (old.ID_SALE_TAXATION <> new.ID_SALE_TAXATION)              /*  tax policy changes       */
       ) then
    begin
      insert into ACCOUNT_POST_CORRECTION
             (ID_ACCOUNT_POST, ID_ACCOUNT, ID_GUEST, IS_QTY_CORRECT, IS_PRICE_CORRECT, ID_SERVICE,
             IS_POSITIVE, OLD_QTY, NEW_QTY, QTY_CORRECT, EXTRA_QTY_CORRECT,
             OLD_PRICE, NEW_PRICE, PRICE_CORRECT, OLD_CURR_POST, OLD_RATE_POST, NEW_CURR_POST, NEW_RATE_POST,
             OLD_POST_NET_VALUE, NEW_POST_NET_VALUE, OLD_POST_GROSS_VALUE, NEW_POST_GROSS_VALUE,
             USER_CORRECT, DATE_POST_ORIG, IS_OPEN, ID_DOC_HEAD, CORRECTION_DATE, DATETIME_CREATED,
             REASON_OF_CORRECT, ID_SALE_TAXATION)
      values (old.ID_ACCOUNT_POST, old.ID_ACCOUNT, old.ID_GUEST, 1,  0, old.ID_SERVICE,
             0         , :old_qty_real, 0, :old_qty_real, 0,
             old.PRICE_POST, old.PRICE_POST, 0, old.curr_post, old.rate_post, old.curr_post, old.rate_post,
             old.POST_NET_VALUE, 0, old.POST_GROSS_VALUE, 0,
             user       ,old.DATE_POST, :IS_OPEN, -1, :sys_date, :datetime_created,
             :reason_of_correct, old.ID_SALE_TAXATION);
      /*  */
      insert into ACCOUNT_POST_CORRECTION
             (ID_ACCOUNT_POST, ID_ACCOUNT, ID_GUEST, IS_QTY_CORRECT, IS_PRICE_CORRECT, ID_SERVICE,
             IS_POSITIVE, OLD_QTY, NEW_QTY, QTY_CORRECT, EXTRA_QTY_CORRECT,
             OLD_PRICE, NEW_PRICE, PRICE_CORRECT,
             OLD_CURR_POST, OLD_RATE_POST, NEW_CURR_POST, NEW_RATE_POST,
             OLD_POST_NET_VALUE, NEW_POST_NET_VALUE, OLD_POST_GROSS_VALUE, NEW_POST_GROSS_VALUE,
             USER_CORRECT, DATE_POST_ORIG, IS_OPEN, ID_DOC_HEAD, CORRECTION_DATE, DATETIME_CREATED,
             REASON_OF_CORRECT, ID_SALE_TAXATION)
      values (old.ID_ACCOUNT_POST, new.ID_ACCOUNT, new.ID_GUEST, 1, 0, new.ID_SERVICE,
             1         , 0, :new_qty_real, :new_qty_real, 0,
             old.PRICE_POST, new.PRICE_POST, 0, old.curr_post, old.rate_post, new.curr_post, new.rate_post,
             0, new.POST_NET_VALUE, 0, new.POST_GROSS_VALUE,
             user       ,new.DATE_POST, :IS_OPEN, -1, :sys_date, :datetime_created,
             :reason_of_correct, new.ID_SALE_TAXATION);

    end else begin /* qty and price/rate corrections   */
      /* log qty change  */
      if (:qty_diff <> 0) then begin
        is_positive = iif(:qty_diff < 0, 0, 1);

        insert into ACCOUNT_POST_CORRECTION
               (ID_ACCOUNT_POST, ID_ACCOUNT, ID_GUEST, IS_QTY_CORRECT, IS_PRICE_CORRECT, ID_SERVICE,
               IS_POSITIVE, OLD_QTY, NEW_QTY, QTY_CORRECT, EXTRA_QTY_CORRECT,
               OLD_PRICE, NEW_PRICE, PRICE_CORRECT, OLD_CURR_POST, OLD_RATE_POST, NEW_CURR_POST, NEW_RATE_POST,
               OLD_POST_NET_VALUE, NEW_POST_NET_VALUE, OLD_POST_GROSS_VALUE, NEW_POST_GROSS_VALUE,
               USER_CORRECT, DATE_POST_ORIG, IS_OPEN, ID_DOC_HEAD, CORRECTION_DATE, DATETIME_CREATED,
               REASON_OF_CORRECT, ID_SALE_TAXATION)
        values (old.ID_ACCOUNT_POST, new.ID_ACCOUNT, new.ID_GUEST, 1, 0, new.ID_SERVICE,
               :is_positive, :old_qty_real, :new_qty_real, abs(:qty_diff), old.QTY_FREE - new.QTY_FREE,
               old.PRICE_POST, old.PRICE_POST, 0 , old.curr_post, old.rate_post, old.curr_post, old.rate_post,
               old.POST_NET_VALUE, iif(:is_value_diff = 0, new.POST_NET_VALUE, old.POST_NET_VALUE),        /* if exists value diffs - new nat values  */
               old.POST_GROSS_VALUE, iif(:is_value_diff = 0, new.POST_GROSS_VALUE, old.POST_GROSS_VALUE),  /* will be stored in next correction record*/
               user, old.DATE_POST, :IS_OPEN, -1, :sys_date, :datetime_created,
               :reason_of_correct, new.ID_SALE_TAXATION);
      end
      /* log price or rate changes */
      if (:is_value_diff = 1) then begin
        is_positive = iif(:value_diff < 0, 0, 1);
        is_price_correct = iif(:price_diff = 0, 0, 1);  -- price or rate difference

        insert into ACCOUNT_POST_CORRECTION
               (ID_ACCOUNT_POST, ID_ACCOUNT, ID_GUEST, IS_QTY_CORRECT, IS_PRICE_CORRECT, IS_POSITIVE,
               ID_SERVICE, OLD_QTY, NEW_QTY, QTY_CORRECT, EXTRA_QTY_CORRECT,
               OLD_PRICE, NEW_PRICE, PRICE_CORRECT,
               OLD_CURR_POST, OLD_RATE_POST, NEW_CURR_POST, NEW_RATE_POST,
               OLD_POST_NET_VALUE, NEW_POST_NET_VALUE, OLD_POST_GROSS_VALUE, NEW_POST_GROSS_VALUE,
               USER_CORRECT, DATE_POST_ORIG, IS_OPEN, ID_DOC_HEAD, CORRECTION_DATE, DATETIME_CREATED,
               REASON_OF_CORRECT, ID_SALE_TAXATION)
        values (old.ID_ACCOUNT_POST, new.ID_ACCOUNT, new.ID_GUEST, 0, :is_price_correct, :is_positive,
               new.ID_SERVICE, :new_qty_real, :new_qty_real, 0, 0,
               old.PRICE_POST, new.PRICE_POST, abs(:value_diff) / new.RATE_POST,
               old.curr_post, old.rate_post, new.curr_post, new.rate_post,
               old.POST_NET_VALUE, new.POST_NET_VALUE, old.POST_GROSS_VALUE, new.POST_GROSS_VALUE,
               user, old.DATE_POST, :IS_OPEN, -1, :sys_date, :datetime_created,
               :reason_of_correct, new.ID_SALE_TAXATION);
      end
    end
  end  /* if ... else  */
end
^

SET TERM ; ^



