set term ^;


create or alter procedure PROFILE_PERSON_CREATEQ$PROFILE (
    PROFILE_NAME type of TPERSON_NAME,
    PHONE type of TPHONE,
    ID_PASSPORT_TYPE type of ID,
    PASS_NUM type of NAME_20,
    EGN type of TTEXT_SHORT,
    COUNTRY_CODE type of TSHORT_CODE,
    CARD_CODE type of TCARD_CODE,
    ID_PERSON_DATA type of ID_REF = null,
    TITLE type of TSHORT_CODE = null,
    ID_HOTEL type of ID_REF = null)
returns (
    NEW_ID_PROFILE type of ID,
    P_DBKEY type of TDB_KEY,
    PD_DBKEY type of TDB_KEY)
AS
declare variable action_text varchar(255);
  declare variable mask varchar(10);
  declare variable first_name varchar(30);
  declare variable surname varchar(30);
  declare variable family_name varchar(30);
  declare variable charset varchar(20);
  declare variable merge_ident_num tname_unicode;
  declare variable merge_ident_num_alt tname_unicode;
begin
  select si.INFO_VALUE
  from   SYSTEM_INFO_GET('DATABASE CHARSET') si
  into   :charset;

  if (:charset = 'WIN1250') then begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1250);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1250);
  end else if (:charset = 'WIN1251') then begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1251);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1251);
  end else if (:charset = 'WIN1252') then begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1252);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1252);
  end else begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1252);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1252);
  end

  new_id_profile = gen_id(gen_profile_id, 1);
  insert into PROFILE (ID_PROFILE, PROFILE_NAME, PROFILE_TYPE, PHONE, IS_ARCHIVE, MERGE_IDENT_NUM, MERGE_IDENT_NUM_ALT)
  values (:new_id_profile, :profile_name, 'PERSON', :phone, 0, trim(:merge_ident_num), trim(:merge_ident_num_alt))
  returning RDB$DB_KEY into :p_dbkey;

  if (:id_person_data is null) then begin
    id_person_data = gen_id(gen_person_data_id, 1);

    if (:ID_HOTEL is not null) then begin
      execute procedure SETTING_GET_VALUE ('DEF_GUEST_MASK', :ID_HOTEL)
      returning_values  :mask;
    end

    select PP.FIRST_NAME,  PP.SURNAME, PP.FAMILY_NAME
    from   PERSON_NAME_PARSE$INT (:PROFILE_NAME) PP
    into   :first_name,  :surname, :family_name;

    insert into PERSON_DATA(ID_PERSON_DATA, PERSON_NAME2, ID_PASSPORT_TYPE, PASS_NUM, EGN, MASK,
           FIRST_NAME, SURNAME, FAMILY_NAME, COUNTRY_CODE)
    values (:id_person_data, :profile_name, :id_passport_type, :pass_num, :egn, :mask,
           :first_name, :surname, :family_name, :country_code)
    returning RDB$DB_KEY into :pd_dbkey;

    execute procedure PERSON_DATA_SET$INT(:p_dbkey, :pd_dbkey, :profile_name, :profile_name, :country_code,
            null, null, :TITLE);
  end

  insert into PROFILE_PERSON (ID_PROFILE, ID_PERSON_DATA, CARD_CODE)
  values (:new_id_profile, :id_person_data, :card_code);

  action_text = f_character(29) || :profile_name;
  execute procedure SYSTEM_LOG_ADD ('PROFILE', :new_id_profile, 'CREATE_PERS_PROFILE', :action_text);
end
^


create or alter procedure PERSON_PASS_DATA_SET$INT (
    PD_DBKEY char(8),
    BIRTH_PLACE varchar(30),
    EGN varchar(30),
    ID_PASSPORT_TYPE integer,
    PASS_NUM varchar(20),
    PASS_DATE date,
    PASS_PLACE varchar(20),
    PASS_EXP_DATE date,
    CITY varchar(100),
    ADDRESS varchar(100),
    SEX char(1),
    MASK varchar(10))
AS
  declare variable id_profile integer;
  declare variable charset varchar(20);
  declare variable merge_ident_num tname_unicode;
  declare variable merge_ident_num_alt tname_unicode;
begin
  /*  Description:
  ||
  */

  update PERSON_DATA PDU
  set    PDU.BIRTH_PLACE = :birth_place,
         PDU.EGN = :egn,
         PDU.ID_PASSPORT_TYPE = :ID_PASSPORT_TYPE,
         PDU.PASS_NUM = :pass_num,
         PDU.PASS_DATE = :pass_date,
         PDU.PASS_EXP_DATE = :pass_exp_date,
         PDU.PASS_PLACE = :pass_place,
         PDU.CITY = :city,
         PDU.ADDRESS = :address,
         PDU.SEX = :sex,
         PDU.MASK = :mask
  where  PDU.RDB$DB_KEY = :pd_dbkey;

  select si.INFO_VALUE
  from   SYSTEM_INFO_GET('DATABASE CHARSET') si
  into   :charset;

  if (:charset = 'WIN1250') then begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1250);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1250);
  end else if (:charset = 'WIN1251') then begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1251);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1251);
  end else if (:charset = 'WIN1252') then begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1252);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1252);
  end else begin
    merge_ident_num = cast(:EGN as varchar(200) character set win1252);
    merge_ident_num_alt = cast(:PASS_NUM as varchar(200) character set win1252);
  end

  for
    select pp.ID_PROFILE
    from   PERSON_DATA pd
           join PROFILE_PERSON pp on pp.ID_PERSON_DATA = pd.ID_PERSON_DATA
    where  PD.RDB$DB_KEY = :pd_dbkey
    into   :id_profile
  do begin
    update PROFILE p
    set    p.MERGE_IDENT_NUM = trim(:merge_ident_num),
           p.MERGE_IDENT_NUM_ALT = trim(:merge_ident_num_alt)
    where  p.ID_PROFILE = :id_profile;
  end 
end
^


create or alter procedure CONTRAGENT_MODIFY$CONTR (
    ID_CONTRAGENT integer,
    CONTRAGENT_NAME varchar(100),
    IS_JURIDICAL_PERSON smallint,
    IDENT_NO varchar(30),
    VAT_REG_NUM varchar(30),
    POST_ADDRESS varchar(100),
    ZIP_CODE varchar(10),
    EMAIL varchar(50),
    MOL varchar(30),
    COUNTRY_CODE varchar(10),
    BANK_NAME varchar(50),
    BIC varchar(30),
    IBAN varchar(30))
returns (
    NEW_ID_CONTRAGENT integer)
AS
declare variable id_profile integer;
  declare variable charset varchar(20);
  declare variable merge_ident_num tname_unicode;
  declare variable merge_ident_num_alt tname_unicode;
begin

  if (:ID_CONTRAGENT is null) then begin
    insert into CONTRAGENTS (CONTRAGENT_NAME, IS_JURIDICAL_PERSON,
           IDENT_NO, VAT_REG_NUM, POST_ADDRESS, ZIP_CODE,
           MOL, BANK_NAME, BIC, IBAN, COUNTRY, EMAIL)
    values (:CONTRAGENT_NAME, :IS_JURIDICAL_PERSON,
           :IDENT_NO, :VAT_REG_NUM, :POST_ADDRESS, :ZIP_CODE,
           :MOL, :BANK_NAME, :BIC, :IBAN, :COUNTRY_CODE, :EMAIL)
    returning ID_CONTRAGENT into :NEW_ID_CONTRAGENT;
  end else begin
    update CONTRAGENTS C
    set    C.CONTRAGENT_NAME = :CONTRAGENT_NAME,
           C.IS_JURIDICAL_PERSON = :IS_JURIDICAL_PERSON,
           C.IDENT_NO = :IDENT_NO,
           C.VAT_REG_NUM = :VAT_REG_NUM,
           C.POST_ADDRESS = :POST_ADDRESS,
           C.ZIP_CODE = :ZIP_CODE,
           C.MOL = :MOL,
           C.COUNTRY = :COUNTRY_CODE,
           C.BANK_NAME = :BANK_NAME,
           C.BIC = :BIC,
           C.IBAN = :IBAN,
           C.EMAIL = :EMAIL
    where  C.ID_CONTRAGENT = :ID_CONTRAGENT;
    NEW_ID_CONTRAGENT = :ID_CONTRAGENT;
  end

  select si.INFO_VALUE
  from   SYSTEM_INFO_GET('DATABASE CHARSET') si
  into   :charset;

  if (:charset = 'WIN1250') then begin
    merge_ident_num = cast(:IDENT_NO as varchar(200) character set win1250);
    merge_ident_num_alt = cast(:VAT_REG_NUM as varchar(200) character set win1250);
  end else if (:charset = 'WIN1251') then begin
    merge_ident_num = cast(:IDENT_NO as varchar(200) character set win1251);
    merge_ident_num_alt = cast(:VAT_REG_NUM as varchar(200) character set win1251);
  end else if (:charset = 'WIN1252') then begin
    merge_ident_num = cast(:IDENT_NO as varchar(200) character set win1252);
    merge_ident_num_alt = cast(:VAT_REG_NUM as varchar(200) character set win1252);
  end else begin
    merge_ident_num = cast(:IDENT_NO as varchar(200) character set win1252);
    merge_ident_num_alt = cast(:VAT_REG_NUM as varchar(200) character set win1252);
  end

  for
    select p.ID_PROFILE
    from   PROFILE p
    where  p.ID_DEF_CONTRAGENT = :ID_CONTRAGENT
           and p.PROFILE_TYPE = 'AGENT'
    into   :id_profile
  do begin
    update PROFILE p
    set    p.MERGE_IDENT_NUM = trim(:merge_ident_num),
           p.MERGE_IDENT_NUM_ALT = trim(:merge_ident_num_alt)
    where  p.ID_PROFILE = :id_profile;
  end
end
^


set term ;^


commit work;


