set term ^;

alter procedure TAR_CALC_EVAL_COMPLEX_RATE$INT (
    INPUT_DETAIL_DBKEY char(8),
    ID_TAR_INPUT integer,
    ID_TAR_CALC integer,
    FOR_DATE date,
    BEDS smallint,
    ID_TYPE_STAY integer,
    IS_MANUAL_TYPE_STAY smallint,
    ID_BOARD integer,
    ID_ROOM_TYPE integer,
    CUSTOM_PRICE numeric(15,4),
    CUSTOM_CURRENCY char(3),
    CHECKIN_DATE date,
    DAY_OF_STAY integer,
    PAYORS integer,
    MINORS integer,
    PERIOD_DEPEND_ON_CHECKIN smallint,
    CNT_DAYS_BP integer,
    CNT_DAYS_BOARD integer,
    BOARD_TYPE_FIND varchar(10),
    TOTAL_GUESTS integer,
    SEQ_NUM integer,
    TARIFF_CURR char(3),
    REAL_RECALC smallint)
returns (
    ERR_CODE integer,
    ERR_TEXT varchar(255),
    OUT_PAYORS integer,
    OUT_MINORS integer)
AS
declare variable id_tar_input_detail integer;
  declare variable price numeric(15,4);
  declare variable auto_stay_type integer;
  declare variable base_per_day numeric(15,4);
  declare variable board_per_day numeric(15,4);
  declare variable guest_board_weight integer;
--  declare variable guest_food varchar(20);
  declare variable match_date date;
  declare variable manual_total_gsts integer;
  declare variable manual_payors integer;
  declare variable manual_minors integer;
  declare variable service_type varchar(20);
  declare variable base_price numeric(15,4);
  declare variable board_price numeric(15,4);
  declare variable food_per_day numeric(15,4);
  declare variable base_price_tar numeric(15,4);
  declare variable custom_cross_rate double precision;
  declare variable ptext varchar(20);
  declare variable id_service integer;
  declare variable qty_post integer;
  declare variable pack_currency char(3);
  declare variable minors_input integer;
  declare variable packet_item integer;
  declare variable price_tariff numeric(15,4);
  declare variable is_inside_base integer;
  declare variable id_bo_bp integer;
  declare variable id_board_include integer;
  declare variable separate_board integer;
  declare variable incomplete_bp integer;
  declare variable id_bo_eg integer;
  declare variable id_eg_price integer;
  declare variable disc_tag varchar(50);
  declare variable id_discount_service integer;
  declare variable var_board_price numeric(15,4);
  declare variable type_stay_code varchar(10);
  declare variable id_hotel integer;
begin
  base_per_day = 0;
  board_per_day = 0;

  select TH.WEIGHT_TAG
  from   MEALS TH
  where  TH.ID_BOARD = :ID_BOARD
  into   :guest_board_weight;
  guest_board_weight = coalesce(:guest_board_weight, 0);

  /*if (:guest_board_weight = 0) then GUEST_FOOD = 'NONE';
  else GUEST_FOOD = 'SOME';*/
  if (:PERIOD_DEPEND_ON_CHECKIN = 1) then MATCH_DATE = :CHECKIN_DATE;
  else MATCH_DATE = :FOR_DATE;

  select st.TYPE_STAY_CODE
  from   TYPE_STAY st
  where  st.ID_TYPE_STAY = :id_type_stay
  into   :type_stay_code;

  if (:IS_MANUAL_TYPE_STAY = 1) then begin
    if (:type_stay_code = 'ALONE') then begin
      MANUAL_TOTAL_GSTS = 1;
      MANUAL_PAYORS = 0;
      MANUAL_MINORS = 0;
    end else if (:type_stay_code = 'BED') then begin
      MANUAL_TOTAL_GSTS = 2;
      MANUAL_PAYORS = 1;
      MANUAL_MINORS = 0;
    end else if (:type_stay_code = 'EXTRA_BED') then begin
      MANUAL_TOTAL_GSTS = :BEDS+1;
      MANUAL_PAYORS = :BEDS;
      MANUAL_MINORS = :MINORS;
    end

    execute procedure TAR_MATCH_BP(:INPUT_DETAIL_DBKEY, :MATCH_DATE, :MANUAL_TOTAL_GSTS, :MANUAL_TOTAL_GSTS,
                      :GUEST_BOARD_WEIGHT, :REAL_RECALC, :DAY_OF_STAY, :CNT_DAYS_BP)
    returning_values  :ERR_CODE, :ERR_TEXT, :ID_BO_BP, :ID_BOARD_INCLUDE,
                      :INCOMPLETE_BP, :BASE_PRICE, :PTEXT, :BASE_PER_DAY;
  end else begin
    execute procedure TAR_MATCH_BP(:INPUT_DETAIL_DBKEY, :MATCH_DATE, :SEQ_NUM, :TOTAL_GUESTS,
                      :GUEST_BOARD_WEIGHT, :REAL_RECALC, :DAY_OF_STAY, :CNT_DAYS_BP)
    returning_values  :ERR_CODE, :ERR_TEXT, :ID_BO_BP, :ID_BOARD_INCLUDE,
                      :INCOMPLETE_BP, :BASE_PRICE, :PTEXT, :BASE_PER_DAY;
  end

  execute procedure sys_dbg('tar base_per_day = '||coalesce(:BASE_PER_DAY, 'null'));
  execute procedure sys_dbg('tar base_price = '||coalesce(:BASE_PRICE, 'null'));
  if (:ERR_CODE < 0) then exit;

  SEPARATE_BOARD = 0;
  if (:REAL_RECALC = 1) then begin
    if (:CUSTOM_PRICE is not null) then begin
      base_price = :CUSTOM_PRICE;
      select tr.ID_HOTEL
      from   TAR_CALC_INPUT_DETAIL$TMP t
             join TARIFA tr on tr.ID_TARIFA = t.ID_TAR_HEAD
      where  t.RDB$DB_KEY = :INPUT_DETAIL_DBKEY
      into   :id_hotel;

      execute procedure CURR_CROSS_RATE_GET(:CUSTOM_CURRENCY, :TARIFF_CURR, current_timestamp, :id_hotel)
      returning_values  :custom_cross_rate;
      base_price_tar = (:custom_cross_rate * :base_price);
      pack_currency = :custom_currency;
    end else begin
      base_price_tar = :base_per_day;
      pack_currency = :tariff_curr;
    end
    /*DEBUG*/
    if (:base_price is null) then exception RATING_EXCEPTION ('4. base price is null');
    if (:base_price_tar is null) then exception RATING_EXCEPTION ('5. base price tar is null');

    select TBP.PACKET_ITEM
    from   TAR_BO_BASE_POSITION TBP
    where  TBP.ID_TAR_BP = :id_bo_bp
    into   :packet_item;

    insert into TAR_CALC_OUTPUT$TMP (
           ID_TAR_INPUT, ID_TAR_CALC, ID_SERVICE, SERVICE_TYPE,
           FOR_DATE, PRICE, PRICE_TARIFF, PRICE_NOTE, QTY_POST, PTEXT,
           CURRENCY, CURRENCY_TARIFF, REQUIRED_ID_PACKET_ITEM)
    values (:ID_TAR_INPUT, :ID_TAR_CALC, -1, 'BASE',
           :FOR_DATE, :BASE_PER_DAY, :BASE_PRICE_TAR, 'BASE', 1, :PTEXT,
           :PACK_CURRENCY, :TARIFF_CURR, :packet_item);
    /* BOARDS */
    var_board_price = 0;
    for
      select TMB.ERR_CODE, TMB.ERR_TEXT, TMB.SEPARATE_BOARD, TMB.BOARD_PRICE, TMB.PTEXT,
             TMB.SERVICE_TYPE, TMB.ID_SERVICE, TMB.FOOD_PER_DAY, TMB.PACKET_ITEM
      from   TAR_MATCH_BOARD(:INPUT_DETAIL_DBKEY, :id_bo_bp, :BOARD_TYPE_FIND,
             :ID_BOARD, :id_board_include, :match_date, :base_per_day,
             :DAY_OF_STAY, :CNT_DAYS_BOARD, :CNT_DAYS_BP, :CUSTOM_PRICE) TMB
      into   :err_code, :err_text, :separate_board, :board_price, :ptext,
             :service_type, :id_service, :food_per_day, :packet_item
    do begin
      if (:ERR_CODE < 0) then exit;

      /*DEBUG*/
      if (:board_price is null) then exception RATING_EXCEPTION ('6. board_price is null');
      if (:id_service is null) then exception RATING_EXCEPTION ('7. id_service is null');

      insert into TAR_CALC_OUTPUT$TMP (
             ID_TAR_INPUT, ID_TAR_CALC, ID_SERVICE, SERVICE_TYPE,
             FOR_DATE, PRICE, PRICE_TARIFF, PRICE_NOTE, QTY_POST, PTEXT,
             CURRENCY, CURRENCY_TARIFF, REQUIRED_ID_PACKET_ITEM)
      values (:ID_TAR_INPUT, :ID_TAR_CALC, :ID_SERVICE, :SERVICE_TYPE,
             :FOR_DATE, :food_per_day, :food_per_day, 'FOOD', 1, :PTEXT,
             :TARIFF_CURR, :TARIFF_CURR, :packet_item);

      -- board_per_day = (:board_per_day + :food_per_day); ???
      var_board_price = :var_board_price + iif(:service_type in ('BASE_BOARD', 'EXTRA_BOARD'), :food_per_day, 0);
    end

    select TID.ID_TAR_INPUT_DETAIL
    from   TAR_CALC_INPUT_DETAIL$TMP TID
    where  tid.RDB$DB_KEY = :INPUT_DETAIL_DBKEY
    into   :id_tar_input_detail;

    execute procedure TAR_SET_VAR('board='||:VAR_BOARD_PRICE, :id_tar_input_detail, :id_tar_calc);
  end

  if (:SEPARATE_BOARD is null) then SEPARATE_BOARD = 0;
  minors_input = :MINORS;

  if (:IS_MANUAL_TYPE_STAY = 1) then begin
    /* ONLY TO FIND THE REAL POSITION PAYORS, MINORS ARE NOT IMPORTANT
    || ALL OTHER GUESTS IN THE ROOM MUST SEE THE MANUAL ONE AS AUTOMATIC. SO
    || HERE WE ARE FINDING HIS AUTOMATIC TYPE STAY.
    */
    execute procedure TAR_MATCH_EG(:INPUT_DETAIL_DBKEY, :ID_BO_BP, :total_guests, :PAYORS, :MINORS, :MATCH_DATE, :INCOMPLETE_BP, :ID_ROOM_TYPE, 1)
    returning_values  :ERR_CODE, :ERR_TEXT, :ID_BO_EG, :ID_EG_PRICE, :PAYORS, :MINORS, :AUTO_STAY_TYPE;
    if (:ERR_CODE < 0) then exit;
    /* REAL PRICE FIND AND EVAL */
    execute procedure TAR_MATCH_MANUAL_EG(:INPUT_DETAIL_DBKEY, :ID_BO_BP, :MANUAL_TOTAL_GSTS, :MANUAL_PAYORS, :MANUAL_MINORS, :MATCH_DATE, :INCOMPLETE_BP, :ID_TYPE_STAY)
    returning_values  :ERR_CODE, :ERR_TEXT, :ID_BO_EG, :ID_EG_PRICE, :AUTO_STAY_TYPE;
    if (:ERR_CODE < 0) then exit;

    if ((:MINORS = :minors_input) and
        (exists (select *
                 from   TYPE_STAY TS
                 where  TS.ID_TYPE_STAY = :ID_TYPE_STAY
                        and TS.TYPE_STAY_CODE = 'EXTRA_BED'))) then begin
      /* TO MANAGE PARAMETER "EXTRA GUEST IN TURN" WHEN MANUAL SET TO ADD */
      MINORS = :MINORS + 1;
      PAYORS = :PAYORS - 1;
    end
  end else begin
    execute procedure TAR_MATCH_EG(:INPUT_DETAIL_DBKEY, :ID_BO_BP, :TOTAL_GUESTS, :PAYORS, :MINORS, :MATCH_DATE, :INCOMPLETE_BP, :ID_ROOM_TYPE, 0)
    returning_values  :ERR_CODE, :ERR_TEXT, :ID_BO_EG, :ID_EG_PRICE, :PAYORS, :MINORS, :AUTO_STAY_TYPE;
    if (:ERR_CODE < 0) then exit;
  end

  update TAR_CALC_INPUT_DETAIL$TMP TID
  set    TID.SEPARATE_BOARD = :SEPARATE_BOARD,
         TID.ID_TYPE_STAY   = :AUTO_STAY_TYPE
  where  TID.RDB$DB_KEY = :INPUT_DETAIL_DBKEY;

  if (:REAL_RECALC = 1) then begin
    if (:CUSTOM_PRICE is null) then begin
      for
        select TSP.SERVICE_TYPE, TSP.PRICE,  TSP.PTEXT, TSP.ERR_CODE, TSP.ID_SERVICE
        from   TAR_PRICES_SET_EG(:ID_TAR_INPUT, :FOR_DATE, :DAY_OF_STAY, :CNT_DAYS_BP,
               :CNT_DAYS_BOARD, :guest_board_weight, :id_bo_eg, :id_eg_price, :base_price) TSP
        into   :service_type, :price,  :ptext, :ERR_CODE, :id_service
      do begin
        if (:ERR_CODE < 0) then exit; -- exception INTERNAL_RATING_EXCEPTION;

        /*DEBUG*/
        if (:PRICE is null) then exception RATING_EXCEPTION ('8. price is null');
        if (:ID_SERVICE is null) then exception RATING_EXCEPTION ('9. id_service is null');

        insert into TAR_CALC_OUTPUT$TMP (
               ID_TAR_INPUT, ID_TAR_CALC, ID_SERVICE, SERVICE_TYPE,
               FOR_DATE, PRICE, PRICE_TARIFF, PRICE_NOTE, QTY_POST, PTEXT,
               CURRENCY, CURRENCY_TARIFF, REQUIRED_ID_PACKET_ITEM)
        values (:ID_TAR_INPUT, :ID_TAR_CALC, :id_service, :service_type,
               :FOR_DATE, :price, :price, 'EG', 1, :ptext,
               :TARIFF_CURR, :TARIFF_CURR, :packet_item);
      end
    end

    for
      select TMT.ID_SERVICE, TMT.SERVICE_TYPE, TMT.PRICE, TMT.PRICE_TARIFF, TMT.QTY_POST,
             TMT.PTEXT, TMT.PACK_CURRENCY, TMT.ERR_CODE, TMT.ERR_TEXT, TMT.PACKET_ITEM, TMT.DISC_TAG
      from   TAR_PRICES_SET_PACKS(:INPUT_DETAIL_DBKEY, :ID_BO_BP, :DAY_OF_STAY, :CUSTOM_PRICE) TMT
      into   :ID_SERVICE, :SERVICE_TYPE, :PRICE, :PRICE_TARIFF, :QTY_POST,
             :PTEXT, :PACK_CURRENCY, :ERR_CODE, :ERR_TEXT, :PACKET_ITEM, :DISC_TAG
    do begin
      if (:ERR_CODE < 0) then exit; --exception INTERNAL_RATING_EXCEPTION;
      /*DEBUG*/
      if (:price_tariff is null) then exception RATING_EXCEPTION ('10. PRICE_TARIFF is null');
      if (:id_service is null) then exception RATING_EXCEPTION ('11. ID_SERVICE is null');

      insert into TAR_CALC_OUTPUT$TMP (
             ID_TAR_INPUT, ID_TAR_CALC, ID_SERVICE, SERVICE_TYPE,
             FOR_DATE, PRICE, PRICE_TARIFF, PRICE_NOTE, QTY_POST, PTEXT,
             CURRENCY, CURRENCY_TARIFF, DISC_TAG, REQUIRED_ID_PACKET_ITEM)
      values (:ID_TAR_INPUT, :ID_TAR_CALC, :ID_SERVICE, :SERVICE_TYPE,
             :FOR_DATE, :PRICE, :PRICE_TARIFF, 'PACK', :QTY_POST, :PTEXT,
             :PACK_CURRENCY, :TARIFF_CURR, :DISC_TAG, :packet_item);

    end

    for
      select TMT.ID_SERVICE, TMT.SERVICE_TYPE, TMT.PRICE, TMT.PTEXT, TMT.ERR_CODE, TMT.ERR_TEXT, TMT.PACKET_ITEM,
             TMT.IS_INSIDE_BASE, TMT.DISC_TAG, TMT.ID_DISCOUNT_SERVICE
      from   TAR_PRICES_SET_TAX(:ID_TAR_INPUT, :INPUT_DETAIL_DBKEY, :ID_BO_BP, :DAY_OF_STAY) tmt
      into   :ID_SERVICE, :SERVICE_TYPE, :PRICE, :PTEXT, :ERR_CODE, :ERR_TEXT, :PACKET_ITEM,
             :IS_INSIDE_BASE, :DISC_TAG, :ID_DISCOUNT_SERVICE
    do begin
      if (:ERR_CODE < 0) then exit; --exception INTERNAL_RATING_EXCEPTION;
      if ((:IS_INSIDE_BASE = 0) and (:CUSTOM_PRICE is not null)) then begin
        -- SKIP IT. SIMPLE DISCOUNTS MUST NOT CHANGE THE CUSTOM PRICE
      end else begin
        /*DEBUG*/
        if (:PRICE is null) then exception RATING_EXCEPTION ('12. price is null');
        if (:ID_SERVICE is null) then exception RATING_EXCEPTION ('13. id_service is null');
        -- !!! Service type and id_service of discount is as object that they discounting but in TAR_CALC_REPOST$INT type and id_service is changed to PACKAGES if discounts have ID_DISCOUNT_SERVICE
        if ((:DISC_TAG = '') or (:DISC_TAG is null)) then DISC_TAG =  'DISC';
        insert into TAR_CALC_OUTPUT$TMP (
               ID_TAR_INPUT, ID_TAR_CALC, ID_SERVICE, SERVICE_TYPE,
               FOR_DATE, PRICE, PRICE_TARIFF, PRICE_NOTE, QTY_POST, PTEXT,
               CURRENCY, CURRENCY_TARIFF, DISC_TAG, ID_DISCOUNT_SERVICE, REQUIRED_ID_PACKET_ITEM)
        values (:ID_TAR_INPUT, :ID_TAR_CALC, :ID_SERVICE, :SERVICE_TYPE,
               :FOR_DATE, :PRICE, :PRICE, 'DISC', 1, :PTEXT,
               :TARIFF_CURR, :TARIFF_CURR, :DISC_TAG, :id_discount_service, :packet_item);

        execute procedure sys_dbg('tar TAR_CALC_EVAL_DATE$INT - :DISC_TAG=' || coalesce(:DISC_TAG, 'null') );
      end
    end
  end

  OUT_MINORS = :MINORS;
  OUT_PAYORS = :PAYORS;
end
^

set term ;^

commit work;
