
insert into ACCOUNT_POST_EXTERNAL (ID_ACCOUNT_POST, EXT_CODE, EXT_USER_VOIDED)
select ap.ID_ACCOUNT_POST, ap.SHORT_CODE, d.EXT_USER_ANUL
from   ACCOUNT_POST AP
       join ACCOUNT_POSTING_DET d on d.ID_ACCOUNT_POST = ap.ID_ACCOUNT_POST
       join SERVICES S on s.ID_SERVICE = ap.ID_SERVICE
where  (ap.SHORT_CODE in ('POS', 'POS_EW', 'TELECOUNT'))
       or (ap.SHORT_CODE = s.EXT_CODE and ap.ID_MASTER_ADDINS_ACC_POST = -1 and ap.SHORT_CODE not in ('RESOURCE', 'PMS'));

commit work;

set term ^;

ALTER trigger PERSON_DATA_BIU0
inactive before insert or update position 0
AS
begin
  -- parse name - for compatibility with other interfaces (cias, etc...)
  if (trim(old.PERSON_NAME) is distinct from trim(new.PERSON_NAME)) then begin
    select PP.FIRST_NAME,  PP.SURNAME, PP.FAMILY_NAME
    from   PERSON_NAME_PARSE$INT (new.PERSON_NAME) PP
    into   new.FIRST_NAME,  new.SURNAME, new.FAMILY_NAME;

    when any do exit;
  end
end
^

ALTER trigger PERSON_DATA_AIU0
inactive after insert or update position 0
AS
begin
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'NAME', old.PERSON_NAME, new.PERSON_NAME);
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'NAME2', old.PERSON_NAME2, new.PERSON_NAME2);
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'EGN', old.EGN, new.EGN);
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'PASS_NUM', old.PASS_NUM, new.PASS_NUM);
end
^

set term ;^

update PERSON_DATA pd
set pd.PERSON_NAME = substring(trim(coalesce(pd.FIRST_NAME || ' ', '')
                                    || coalesce(pd.SURNAME || ' ', '')
                                    || coalesce(pd.FAMILY_NAME, '')) from 1 for 100);
commit work;

set term ^;

ALTER trigger PERSON_DATA_BIU0
active before insert or update position 0
AS
begin
  -- parse name - for compatibility with other interfaces (cias, etc...)
  if (trim(old.PERSON_NAME) is distinct from trim(new.PERSON_NAME)) then begin
    select PP.FIRST_NAME,  PP.SURNAME, PP.FAMILY_NAME
    from   PERSON_NAME_PARSE$INT (new.PERSON_NAME) PP
    into   new.FIRST_NAME,  new.SURNAME, new.FAMILY_NAME;

    when any do exit;
  end
end
^

ALTER trigger PERSON_DATA_AIU0
active after insert or update position 0
AS
begin
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'NAME', old.PERSON_NAME, new.PERSON_NAME);
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'NAME2', old.PERSON_NAME2, new.PERSON_NAME2);
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'EGN', old.EGN, new.EGN);
  execute procedure PROFILE_SEARCH_TAG_MODIFY(null, new.ID_PERSON_DATA, null, 'PASS_NUM', old.PASS_NUM, new.PASS_NUM);
end
^

alter procedure PROFILE_SEARCH_TAG_MODIFY (
    ID_PROFILE type of ID,
    ID_PERSON_DATA type of ID,
    ID_CONTRAGENT type of ID,
    TAG_NAME type of TSEARCH_CODE,
    OLD_VALUE type of TDESCRIPTION_SHORT,
    NEW_VALUE type of TDESCRIPTION_SHORT)
AS
declare variable tag_value type of TNAME_UNICODE;
  declare variable item varchar(255);
begin
  NEW_VALUE = trim(coalesce(:NEW_VALUE, ''));
  OLD_VALUE = trim(coalesce(:OLD_VALUE, ''));
  if ((char_length(:NEW_VALUE) = 0) or (:NEW_VALUE is distinct from :OLD_VALUE)) then begin
/*$$IBEC$$     delete from PROFILE_SEARCH_TAG st
    where  st.ID_PROFILE is not distinct from :ID_PROFILE
           and st.ID_PERSON_DATA is not distinct from :ID_PERSON_DATA
           and st.ID_CONTRAGENT is not distinct from :ID_CONTRAGENT
           and st.TAG_NAME = :TAG_NAME
    PLAN (ST INDEX (PROFILE_SEARCH_TAG_IDX1));
 $$IBEC$$*/
    if (char_length(:NEW_VALUE) > 0) then begin
      if (:TAG_NAME in ('NAME', 'NAME2')) then begin
        for
          select distinct sp.sub_name
          from   SYSTEM_PARSE_NAME$INT(:NEW_VALUE) SP
          where  char_length(sp.SUB_NAME) >= 3
                 --not (char_length(sp.SUB_NAME) = 1 and ascii_val(sp.SUB_NAME) < 65)
          into   :item
        do begin
          select substring(ss.UNICODE_STRING from 1 for 200)
          from   SYSTEM_STRING_TO_UNICODE(:item) ss
          into   :tag_value;

          insert into PROFILE_SEARCH_TAG(ID_PROFILE, ID_PERSON_DATA, ID_CONTRAGENT, TAG_NAME, TAG_VALUE)
          values (:ID_PROFILE, :ID_PERSON_DATA, :ID_CONTRAGENT, :TAG_NAME, :tag_value);
        end

        select substring(ss.UNICODE_STRING from 1 for 200)
        from   SYSTEM_STRING_TO_UNICODE(:NEW_VALUE) ss
        into   :tag_value;

        insert into PROFILE_SEARCH_TAG(ID_PROFILE, ID_PERSON_DATA, ID_CONTRAGENT, TAG_NAME, TAG_VALUE)
        values (:ID_PROFILE, :ID_PERSON_DATA, :ID_CONTRAGENT, :TAG_NAME, :tag_value);
      end else begin
        select substring(ss.UNICODE_STRING from 1 for 200)
        from   SYSTEM_STRING_TO_UNICODE(:NEW_VALUE) ss
        into   :tag_value;

        insert into PROFILE_SEARCH_TAG(ID_PROFILE, ID_PERSON_DATA, ID_CONTRAGENT, TAG_NAME, TAG_VALUE)
        values (:ID_PROFILE, :ID_PERSON_DATA, :ID_CONTRAGENT, :TAG_NAME, :tag_value);
      end
    end
  end

  when any do begin
    if (:ID_PROFILE is not null) then savedbg('>> Error of PROFILE_SEARCH_TAG_MODIFY! ' || 'ID_PROFILE: ' || :ID_PROFILE || '; TAG_NAME: ' || :TAG_NAME || '; NEW_VALUE: ' || coalesce(:NEW_VALUE, '<null>'));
    else if (:ID_PERSON_DATA is not null) then savedbg('>> Error of PROFILE_SEARCH_TAG_MODIFY! ' || 'ID_PERSON_DATA: ' || :ID_PERSON_DATA || '; TAG_NAME: ' || :TAG_NAME || '; NEW_VALUE: ' || coalesce(:NEW_VALUE, '<null>'));
    else if (:ID_CONTRAGENT is not null) then savedbg('>> Error of PROFILE_SEARCH_TAG_MODIFY! ' || 'ID_CONTRAGENT: ' || :ID_CONTRAGENT || '; TAG_NAME: ' || :TAG_NAME || '; NEW_VALUE: ' || coalesce(:NEW_VALUE, '<null>'));
  end
end
^

set term ;^

RECONNECT;

delete from LISTS_ITEMS li
where li.LIST_GROUP = 'person_title'
      and coalesce(ascii_val(nullif(trim(li.LIST_ITEM), '') ), 0) < 65;

commit work;

ALTER INDEX PROFILE_SEARCH_TAG_IDX1 INACTIVE;
ALTER INDEX PST_TAG_NAME_IDX INACTIVE;
ALTER INDEX PST_TAG_VALUE_IDX INACTIVE;

ALTER TABLE PROFILE_SEARCH_TAG DROP CONSTRAINT FK_PST_CONTRAGENT;
ALTER TABLE PROFILE_SEARCH_TAG DROP CONSTRAINT FK_PST_PERSON_DATA;
ALTER TABLE PROFILE_SEARCH_TAG DROP CONSTRAINT FK_PST_PROFILE;

RECONNECT;

execute procedure PROFILE_SEARCH_TAGS_FILL;

commit work;

RECONNECT;

ALTER INDEX PROFILE_SEARCH_TAG_IDX1 ACTIVE;
ALTER INDEX PST_TAG_NAME_IDX ACTIVE;
ALTER INDEX PST_TAG_VALUE_IDX ACTIVE;

ALTER TABLE PROFILE_SEARCH_TAG ADD CONSTRAINT FK_PST_CONTRAGENT FOREIGN KEY (ID_CONTRAGENT) REFERENCES CONTRAGENTS (ID_CONTRAGENT) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE PROFILE_SEARCH_TAG ADD CONSTRAINT FK_PST_PERSON_DATA FOREIGN KEY (ID_PERSON_DATA) REFERENCES PERSON_DATA (ID_PERSON_DATA) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE PROFILE_SEARCH_TAG ADD CONSTRAINT FK_PST_PROFILE FOREIGN KEY (ID_PROFILE) REFERENCES PROFILE (ID_PROFILE) ON DELETE CASCADE ON UPDATE CASCADE;

RECONNECT;

set term ^;

alter procedure PROFILE_SEARCH_TAG_MODIFY (
    ID_PROFILE type of ID,
    ID_PERSON_DATA type of ID,
    ID_CONTRAGENT type of ID,
    TAG_NAME type of TSEARCH_CODE,
    OLD_VALUE type of TDESCRIPTION_SHORT,
    NEW_VALUE type of TDESCRIPTION_SHORT)
AS
declare variable tag_value type of TNAME_UNICODE;
  declare variable item varchar(255);
begin
  NEW_VALUE = trim(coalesce(:NEW_VALUE, ''));
  OLD_VALUE = trim(coalesce(:OLD_VALUE, ''));
  if ((char_length(:NEW_VALUE) = 0) or (:NEW_VALUE is distinct from :OLD_VALUE)) then begin
    delete from PROFILE_SEARCH_TAG st
    where  st.ID_PROFILE is not distinct from :ID_PROFILE
           and st.ID_PERSON_DATA is not distinct from :ID_PERSON_DATA
           and st.ID_CONTRAGENT is not distinct from :ID_CONTRAGENT
           and st.TAG_NAME = :TAG_NAME
    PLAN (ST INDEX (PROFILE_SEARCH_TAG_IDX1));

    if (char_length(:NEW_VALUE) > 0) then begin
      if (:TAG_NAME in ('NAME', 'NAME2')) then begin
        for
          select distinct sp.sub_name
          from   SYSTEM_PARSE_NAME$INT(:NEW_VALUE) SP
          where  char_length(sp.SUB_NAME) >= 3
                 --not (char_length(sp.SUB_NAME) = 1 and ascii_val(sp.SUB_NAME) < 65)
          into   :item
        do begin
          select substring(ss.UNICODE_STRING from 1 for 200)
          from   SYSTEM_STRING_TO_UNICODE(:item) ss
          into   :tag_value;

          insert into PROFILE_SEARCH_TAG(ID_PROFILE, ID_PERSON_DATA, ID_CONTRAGENT, TAG_NAME, TAG_VALUE)
          values (:ID_PROFILE, :ID_PERSON_DATA, :ID_CONTRAGENT, :TAG_NAME, :tag_value);
        end

        select substring(ss.UNICODE_STRING from 1 for 200)
        from   SYSTEM_STRING_TO_UNICODE(:NEW_VALUE) ss
        into   :tag_value;

        insert into PROFILE_SEARCH_TAG(ID_PROFILE, ID_PERSON_DATA, ID_CONTRAGENT, TAG_NAME, TAG_VALUE)
        values (:ID_PROFILE, :ID_PERSON_DATA, :ID_CONTRAGENT, :TAG_NAME, :tag_value);
      end else begin
        select substring(ss.UNICODE_STRING from 1 for 200)
        from   SYSTEM_STRING_TO_UNICODE(:NEW_VALUE) ss
        into   :tag_value;

        insert into PROFILE_SEARCH_TAG(ID_PROFILE, ID_PERSON_DATA, ID_CONTRAGENT, TAG_NAME, TAG_VALUE)
        values (:ID_PROFILE, :ID_PERSON_DATA, :ID_CONTRAGENT, :TAG_NAME, :tag_value);
      end
    end
  end

  when any do begin
    if (:ID_PROFILE is not null) then savedbg('>> Error of PROFILE_SEARCH_TAG_MODIFY! ' || 'ID_PROFILE: ' || :ID_PROFILE || '; TAG_NAME: ' || :TAG_NAME || '; NEW_VALUE: ' || coalesce(:NEW_VALUE, '<null>'));
    else if (:ID_PERSON_DATA is not null) then savedbg('>> Error of PROFILE_SEARCH_TAG_MODIFY! ' || 'ID_PERSON_DATA: ' || :ID_PERSON_DATA || '; TAG_NAME: ' || :TAG_NAME || '; NEW_VALUE: ' || coalesce(:NEW_VALUE, '<null>'));
    else if (:ID_CONTRAGENT is not null) then savedbg('>> Error of PROFILE_SEARCH_TAG_MODIFY! ' || 'ID_CONTRAGENT: ' || :ID_CONTRAGENT || '; TAG_NAME: ' || :TAG_NAME || '; NEW_VALUE: ' || coalesce(:NEW_VALUE, '<null>'));
  end
end
^

set term ;^

update BLOCK_ROOM b
set    b.ID_HOTEL = (select r.ID_HOTEL
                     from   ROOMS r
                     where  r.ID_ROOM = b.ID_ROOM);

commit work;

delete from SUMMARY_REBUILD_DATE;

commit work;

insert into SUMMARY_REBUILD_DATE (id_hotel, rebuild_date, id_reserv)
select rd.ID_HOTEL, p.OUT_DATE, rd.ID_RESERV_DETAIL
from   RESERV_CHANGE_SP sp
       join RESERV_DETAIl rd on rd.ID_RESERV_DETAIL = sp.ID_RESERV_DETAIL
       left outer join SYSTEM_LOOP_PERIOD(sp.FROM_DATE, sp.TO_DATE) p on 1=1;

commit work;

insert into SYSTEM_CACHE (NOM_CODE, DATETIME_MODIFIED)
select first 1 'end_work', ew.END_WORK_DATETIME
from end_works ew
where ew.END_WORK_KIND = '3'
order by ew.END_WORK_DATETIME desc;

commit work;

update or insert into CHANNEL (NAME) values ('ClockWRS') matching (NAME);
update or insert into CHANNEL_SCHEDULE$EXC(CHANNEL_NAME) values ('ClockWRS') matching (CHANNEL_NAME);

commit work;
